@@ -235,59 +235,71 @@ def cleanSourceDirectory(source_dir):
 
 
 def pickSourceFilenames(source_dir, modules):
+    """ Pick the names for the C files of each module.
+
+        Args:
+            source_dir - the directory to put the module sources will be put into
+            modules    - all the modules to build.
+
+        Returns:
+            Dictionary mapping modules to filenames in source_dir.
+
+        Notes:
+            These filenames can collide, due to e.g. mixed case usage, or there
+            being duplicate copies, e.g. a package named the same as the main
+            binary.
+
+            Conflicts are resolved by appending @<number> with a count in the
+            list of sorted modules. We try to be reproducible here, so we get
+            still good caching for external tools.
+    """
+
     collision_filenames = set()
-    seen_filenames = set()
 
-    # Our output.
-    module_filenames = {}
-
-    def getFilenames(module):
-        base_filename = os.path.join(
-            source_dir,
-            "module." + module.getFullName()
-            if not module.isInternalModule()
-            else module.getFullName(),
-        )
+    def _getModuleFilenames(module):
+        base_filename = os.path.join(source_dir, "module." + module.getFullName())
 
         # Note: Could detect if the file system is cases sensitive in source_dir
         # or not, but that's probably not worth the effort. False positives do
-        # no harm at all.
-        collision_filename = os.path.normcase(base_filename)
+        # no harm at all. We cannot use normcase, as macOS is not using one that
+        # will tell us the truth.
+        collision_filename = base_filename.lower()
 
         return base_filename, collision_filename
 
+    seen_filenames = set()
+
     # First pass, check for collisions.
     for module in modules:
         if module.isPythonShlibModule():
             continue
 
-        base_filename = base_filename, collision_filename = getFilenames(module)
+        base_filename = base_filename, collision_filename = _getModuleFilenames(module)
 
         if collision_filename in seen_filenames:
             collision_filenames.add(collision_filename)
 
         seen_filenames.add(collision_filename)
 
-    # Count up for colliding filenames.
+    # Our output.
+    module_filenames = {}
+
+    # Count up for colliding filenames as we go.
     collision_counts = {}
 
     # Second pass, this time sorted, so we get deterministic results. We will
-    # apply an @1/@2 to disambiguate the filenames.
+    # apply an "@1"/"@2",... to disambiguate the filenames.
     for module in sorted(modules, key=lambda x: x.getFullName()):
         if module.isPythonShlibModule():
             continue
 
-        base_filename = base_filename, collision_filename = getFilenames(module)
+        base_filename = base_filename, collision_filename = _getModuleFilenames(module)
 
         if collision_filename in collision_filenames:
             collision_counts[collision_filename] = (
                 collision_counts.get(collision_filename, 0) + 1
             )
-            hash_suffix = "@%d" % collision_counts[collision_filename]
-        else:
-            hash_suffix = ""
-
-        base_filename += hash_suffix
+            base_filename += "@%d" % collision_counts[collision_filename]
 
         module_filenames[module] = base_filename + ".c"
 
@@ -417,7 +429,8 @@ def makeSourceDirectory(main_module):
                 (module.getFilename(), target_filename, module.getPackage())
             )
         elif module.isUncompiledPythonModule():
-            pass
+            if Options.isShowInclusion():
+                info("Included uncompiled module '%s'." % module.getFullName())
         else:
             assert False, module
 
