@@ -156,6 +156,7 @@ from .IntegerCodes import (
     generateBuiltinLong2Code,
 )
 from .IteratorCodes import (
+    generateBuiltinAnyCode,
     generateBuiltinIter1Code,
     generateBuiltinIter2Code,
     generateBuiltinIterForUnpackCode,
@@ -450,9 +451,9 @@ def prepareModuleCode(global_context, module, module_name):
         temp_variables=module.getTempVariables(),
         outline_variables=module.getOutlineLocalVariables(),
         is_main_module=module.isMainModule(),
-        is_top_module=module.isTopModule(),
         is_internal_module=module.isInternalModule(),
         is_package=module.isCompiledPythonPackage(),
+        is_top=module.isTopModule(),
         context=context,
     )
 
@@ -522,6 +523,7 @@ setExpressionDispatchDict(
         "EXPRESSION_BUILTIN_TUPLE": generateBuiltinTupleCode,
         "EXPRESSION_BUILTIN_LIST": generateBuiltinListCode,
         "EXPRESSION_BUILTIN_SET": generateBuiltinSetCode,
+        "EXPRESSION_BUILTIN_ANY": generateBuiltinAnyCode,
         "EXPRESSION_BUILTIN_FROZENSET": generateBuiltinFrozensetCode,
         "EXPRESSION_BUILTIN_DICT": generateBuiltinDictCode,
         "EXPRESSION_BUILTIN_LOCALS_COPY": generateBuiltinLocalsCode,
