@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,34 +26,28 @@ from nuitka.Options import isDebug
 from .CodeHelpers import (
     generateChildExpressionsCode,
     generateExpressionCode,
-    withObjectCodeTemporaryAssignment
+    withObjectCodeTemporaryAssignment,
 )
 from .ErrorCodes import getFrameVariableTypeDescriptionCode
 from .LabelCodes import getGotoCode
-from .LineNumberCodes import (
-    emitErrorLineNumberUpdateCode,
-    getErrorLineNumberUpdateCode
-)
+from .LineNumberCodes import emitErrorLineNumberUpdateCode, getErrorLineNumberUpdateCode
 from .PythonAPICodes import getReferenceExportCode
 
 
 def generateReraiseCode(statement, emit, context):
     old_source_ref = context.setCurrentSourceCodeReference(
-        value = statement.getCompatibleSourceReference()
+        value=statement.getCompatibleSourceReference()
     )
 
-    getReRaiseExceptionCode(
-        emit    = emit,
-        context = context
-    )
+    getReRaiseExceptionCode(emit=emit, context=context)
 
     context.setCurrentSourceCodeReference(old_source_ref)
 
 
 def generateRaiseCode(statement, emit, context):
-    exception_type  = statement.getExceptionType()
+    exception_type = statement.getExceptionType()
     exception_value = statement.getExceptionValue()
-    exception_tb    = statement.getExceptionTrace()
+    exception_tb = statement.getExceptionTrace()
     exception_cause = statement.getExceptionCause()
 
     # Exception cause is only possible with simple raise form.
@@ -62,127 +56,126 @@ def generateRaiseCode(statement, emit, context):
         assert exception_value is None
         assert exception_tb is None
 
-        raise_type_name  = context.allocateTempName("raise_type")
+        raise_type_name = context.allocateTempName("raise_type")
 
         generateExpressionCode(
-            to_name    = raise_type_name,
-            expression = exception_type,
-            emit       = emit,
-            context    = context
+            to_name=raise_type_name,
+            expression=exception_type,
+            emit=emit,
+            context=context,
         )
 
-        raise_cause_name  = context.allocateTempName("raise_cause")
+        raise_cause_name = context.allocateTempName("raise_cause")
 
         generateExpressionCode(
-            to_name    = raise_cause_name,
-            expression = exception_cause,
-            emit       = emit,
-            context    = context
+            to_name=raise_cause_name,
+            expression=exception_cause,
+            emit=emit,
+            context=context,
         )
 
-        old_source_ref = context.setCurrentSourceCodeReference(exception_cause.getSourceReference())
+        old_source_ref = context.setCurrentSourceCodeReference(
+            exception_cause.getSourceReference()
+        )
 
         _getRaiseExceptionWithCauseCode(
-            raise_type_name  = raise_type_name,
-            raise_cause_name = raise_cause_name,
-            emit             = emit,
-            context          = context
+            raise_type_name=raise_type_name,
+            raise_cause_name=raise_cause_name,
+            emit=emit,
+            context=context,
         )
 
         context.setCurrentSourceCodeReference(old_source_ref)
     elif exception_type is None:
         assert False, statement
     elif exception_value is None and exception_tb is None:
-        raise_type_name  = context.allocateTempName("raise_type")
+        raise_type_name = context.allocateTempName("raise_type")
 
         generateExpressionCode(
-            to_name    = raise_type_name,
-            expression = exception_type,
-            emit       = emit,
-            context    = context
+            to_name=raise_type_name,
+            expression=exception_type,
+            emit=emit,
+            context=context,
         )
 
         old_source_ref = context.setCurrentSourceCodeReference(
-            value = exception_type.getCompatibleSourceReference()
+            value=exception_type.getCompatibleSourceReference()
         )
 
         _getRaiseExceptionWithTypeCode(
-            raise_type_name = raise_type_name,
-            emit            = emit,
-            context         = context
+            raise_type_name=raise_type_name, emit=emit, context=context
         )
 
         context.setCurrentSourceCodeReference(old_source_ref)
     elif exception_tb is None:
-        raise_type_name  = context.allocateTempName("raise_type")
+        raise_type_name = context.allocateTempName("raise_type")
 
         generateExpressionCode(
-            to_name    = raise_type_name,
-            expression = exception_type,
-            emit       = emit,
-            context    = context
+            to_name=raise_type_name,
+            expression=exception_type,
+            emit=emit,
+            context=context,
         )
 
-        raise_value_name  = context.allocateTempName("raise_value")
+        raise_value_name = context.allocateTempName("raise_value")
 
         generateExpressionCode(
-            to_name    = raise_value_name,
-            expression = exception_value,
-            emit       = emit,
-            context    = context
+            to_name=raise_value_name,
+            expression=exception_value,
+            emit=emit,
+            context=context,
         )
 
-        old_source_ref = context.setCurrentSourceCodeReference(exception_value.getSourceReference())
-
-        context.setCurrentSourceCodeReference(
-            statement.getCompatibleSourceReference()
+        old_source_ref = context.setCurrentSourceCodeReference(
+            exception_value.getSourceReference()
         )
 
+        context.setCurrentSourceCodeReference(statement.getCompatibleSourceReference())
+
         _getRaiseExceptionWithValueCode(
-            raise_type_name  = raise_type_name,
-            raise_value_name = raise_value_name,
-            implicit         = statement.isStatementRaiseExceptionImplicit(),
-            emit             = emit,
-            context          = context
+            raise_type_name=raise_type_name,
+            raise_value_name=raise_value_name,
+            implicit=statement.isStatementRaiseExceptionImplicit(),
+            emit=emit,
+            context=context,
         )
 
         context.setCurrentSourceCodeReference(old_source_ref)
     else:
-        raise_type_name  = context.allocateTempName("raise_type")
+        raise_type_name = context.allocateTempName("raise_type")
 
         generateExpressionCode(
-            to_name    = raise_type_name,
-            expression = exception_type,
-            emit       = emit,
-            context    = context
+            to_name=raise_type_name,
+            expression=exception_type,
+            emit=emit,
+            context=context,
         )
 
-        raise_value_name  = context.allocateTempName("raise_value")
+        raise_value_name = context.allocateTempName("raise_value")
 
         generateExpressionCode(
-            to_name    = raise_value_name,
-            expression = exception_value,
-            emit       = emit,
-            context    = context
+            to_name=raise_value_name,
+            expression=exception_value,
+            emit=emit,
+            context=context,
         )
 
         raise_tb_name = context.allocateTempName("raise_tb")
 
         generateExpressionCode(
-            to_name    = raise_tb_name,
-            expression = exception_tb,
-            emit       = emit,
-            context    = context
+            to_name=raise_tb_name, expression=exception_tb, emit=emit, context=context
         )
 
-        old_source_ref = context.setCurrentSourceCodeReference(exception_tb.getSourceReference())
+        old_source_ref = context.setCurrentSourceCodeReference(
+            exception_tb.getSourceReference()
+        )
 
         _getRaiseExceptionWithTracebackCode(
-            raise_type_name  = raise_type_name,
-            raise_value_name = raise_value_name,
-            raise_tb_name    = raise_tb_name,
-            emit             = emit,
-            context          = context
+            raise_type_name=raise_type_name,
+            raise_value_name=raise_value_name,
+            raise_tb_name=raise_tb_name,
+            emit=emit,
+            context=context,
         )
 
         context.setCurrentSourceCodeReference(old_source_ref)
@@ -190,9 +183,7 @@ def generateRaiseCode(statement, emit, context):
 
 def generateRaiseExpressionCode(to_name, expression, emit, context):
     arg_names = generateChildExpressionsCode(
-        expression = expression,
-        emit       = emit,
-        context    = context
+        expression=expression, emit=emit, context=context
     )
 
     # Missed optimization opportunity, please report it, this should not
@@ -202,29 +193,32 @@ def generateRaiseExpressionCode(to_name, expression, emit, context):
         # it handles cases where the value is not in locals dict properly.
 
         parent = expression.parent
-        assert parent.isExpressionSideEffects() or \
-               parent.isExpressionConditional() or \
-               parent.isExpressionLocalsVariableRefORFallback(), \
-               (expression, expression.parent, expression.asXmlText())
+        assert (
+            parent.isExpressionSideEffects()
+            or parent.isExpressionConditional()
+            or parent.isExpressionLocalsVariableRefORFallback()
+        ), (expression, expression.parent, expression.asXmlText())
 
-    with withObjectCodeTemporaryAssignment(to_name, "raise_exception_result", expression, emit, context) \
-      as value_name:
+    with withObjectCodeTemporaryAssignment(
+        to_name, "raise_exception_result", expression, emit, context
+    ) as value_name:
 
         # That's how we indicate exception to the surrounding world.
         emit("%s = NULL;" % value_name)
 
         _getRaiseExceptionWithValueCode(
-            raise_type_name  = arg_names[0],
-            raise_value_name = arg_names[1],
-            implicit         = True,
-            emit             = emit,
-            context          = context
+            raise_type_name=arg_names[0],
+            raise_value_name=arg_names[1],
+            implicit=True,
+            emit=emit,
+            context=context,
         )
 
 
 def getReRaiseExceptionCode(emit, context):
-    exception_type, exception_value, exception_tb, exception_lineno = \
-      context.variable_storage.getExceptionVariableDescriptions()
+    exception_type, exception_value, exception_tb, exception_lineno = (
+        context.variable_storage.getExceptionVariableDescriptions()
+    )
 
     keeper_variables = context.getExceptionKeeperVariables()
 
@@ -236,13 +230,13 @@ if (unlikely( %(bool_res_name)s == false ))
 {
     %(update_code)s
 }
-""" % {
-                "exception_type"  : exception_type,
-                "exception_value" : exception_value,
-                "exception_tb"    : exception_tb,
-                "bool_res_name"   : context.getBoolResName(),
-                "update_code"     : getErrorLineNumberUpdateCode(context)
-
+"""
+            % {
+                "exception_type": exception_type,
+                "exception_value": exception_value,
+                "exception_tb": exception_tb,
+                "bool_res_name": context.getBoolResName(),
+                "update_code": getErrorLineNumberUpdateCode(context),
             }
         )
 
@@ -252,17 +246,18 @@ if (unlikely( %(bool_res_name)s == false ))
             emit(
                 """\
 if (%(exception_tb)s && %(exception_tb)s->tb_frame == &%(frame_identifier)s->m_frame) \
-%(frame_identifier)s->m_frame.f_lineno = %(exception_tb)s->tb_lineno;""" % {
-                    "exception_tb"     : exception_tb,
-                    "frame_identifier" : context.getFrameHandle()
+%(frame_identifier)s->m_frame.f_lineno = %(exception_tb)s->tb_lineno;"""
+                % {
+                    "exception_tb": exception_tb,
+                    "frame_identifier": context.getFrameHandle(),
                 }
             )
 
-            emit(
-                getFrameVariableTypeDescriptionCode(context)
-            )
+            emit(getFrameVariableTypeDescriptionCode(context))
     else:
-        keeper_type, keeper_value, keeper_tb, keeper_lineno = context.getExceptionKeeperVariables()
+        keeper_type, keeper_value, keeper_tb, keeper_lineno = (
+            context.getExceptionKeeperVariables()
+        )
 
         emit(
             """\
@@ -271,53 +266,41 @@ if (%(exception_tb)s && %(exception_tb)s->tb_frame == &%(frame_identifier)s->m_f
 %(exception_value)s = %(keeper_value)s;
 %(exception_tb)s = %(keeper_tb)s;
 %(exception_lineno)s = %(keeper_lineno)s;
-""" %  {
-            "exception_type"   : exception_type,
-            "exception_value"  : exception_value,
-            "exception_tb"     : exception_tb,
-            "exception_lineno" : exception_lineno,
-            "keeper_type"      : keeper_type,
-            "keeper_value"     : keeper_value,
-            "keeper_tb"        : keeper_tb,
-            "keeper_lineno"    : keeper_lineno
+"""
+            % {
+                "exception_type": exception_type,
+                "exception_value": exception_value,
+                "exception_tb": exception_tb,
+                "exception_lineno": exception_lineno,
+                "keeper_type": keeper_type,
+                "keeper_value": keeper_value,
+                "keeper_tb": keeper_tb,
+                "keeper_lineno": keeper_lineno,
             }
         )
 
     getGotoCode(context.getExceptionEscape(), emit)
 
 
-def _getRaiseExceptionWithCauseCode(raise_type_name, raise_cause_name, emit,
-                                   context):
-    exception_type, exception_value, exception_tb, _exception_lineno = \
-      context.variable_storage.getExceptionVariableDescriptions()
-
-    emit(
-        "%s = %s;" % (
-            exception_type,
-            raise_type_name
-        )
+def _getRaiseExceptionWithCauseCode(raise_type_name, raise_cause_name, emit, context):
+    exception_type, exception_value, exception_tb, _exception_lineno = (
+        context.variable_storage.getExceptionVariableDescriptions()
     )
+
+    emit("%s = %s;" % (exception_type, raise_type_name))
     getReferenceExportCode(raise_type_name, emit, context)
 
-    emit(
-        "%s = NULL;" % exception_value
-    )
+    emit("%s = NULL;" % exception_value)
 
     getReferenceExportCode(raise_cause_name, emit, context)
 
     emitErrorLineNumberUpdateCode(emit, context)
     emit(
-        "RAISE_EXCEPTION_WITH_CAUSE( &%s, &%s, &%s, %s );" % (
-            exception_type,
-            exception_value,
-            exception_tb,
-            raise_cause_name
-        )
+        "RAISE_EXCEPTION_WITH_CAUSE( &%s, &%s, &%s, %s );"
+        % (exception_type, exception_value, exception_tb, raise_cause_name)
     )
 
-    emit(
-        getFrameVariableTypeDescriptionCode(context)
-    )
+    emit(getFrameVariableTypeDescriptionCode(context))
 
     getGotoCode(context.getExceptionEscape(), emit)
 
@@ -328,29 +311,20 @@ def _getRaiseExceptionWithCauseCode(raise_type_name, raise_cause_name, emit,
 
 
 def _getRaiseExceptionWithTypeCode(raise_type_name, emit, context):
-    exception_type, exception_value, exception_tb, _exception_lineno = \
-      context.variable_storage.getExceptionVariableDescriptions()
-
-    emit(
-        "%s = %s;" % (
-            exception_type,
-            raise_type_name
-        )
+    exception_type, exception_value, exception_tb, _exception_lineno = (
+        context.variable_storage.getExceptionVariableDescriptions()
     )
+
+    emit("%s = %s;" % (exception_type, raise_type_name))
     getReferenceExportCode(raise_type_name, emit, context)
 
     emitErrorLineNumberUpdateCode(emit, context)
     emit(
-        "RAISE_EXCEPTION_WITH_TYPE( &%s, &%s, &%s );" % (
-            exception_type,
-            exception_value,
-            exception_tb
-        )
+        "RAISE_EXCEPTION_WITH_TYPE( &%s, &%s, &%s );"
+        % (exception_type, exception_value, exception_tb)
     )
 
-    emit(
-        getFrameVariableTypeDescriptionCode(context)
-    )
+    emit(getFrameVariableTypeDescriptionCode(context))
 
     getGotoCode(context.getExceptionEscape(), emit)
 
@@ -358,39 +332,30 @@ def _getRaiseExceptionWithTypeCode(raise_type_name, emit, context):
         context.removeCleanupTempName(raise_type_name)
 
 
-def _getRaiseExceptionWithValueCode(raise_type_name, raise_value_name, implicit,
-                                   emit, context):
-    exception_type, exception_value, exception_tb, _exception_lineno = \
-      context.variable_storage.getExceptionVariableDescriptions()
+def _getRaiseExceptionWithValueCode(
+    raise_type_name, raise_value_name, implicit, emit, context
+):
+    exception_type, exception_value, exception_tb, _exception_lineno = (
+        context.variable_storage.getExceptionVariableDescriptions()
+    )
 
-    emit(
-        "%s = %s;" % (
-            exception_type,
-            raise_type_name
-        )
-    )
+    emit("%s = %s;" % (exception_type, raise_type_name))
     getReferenceExportCode(raise_type_name, emit, context)
-    emit(
-        "%s = %s;" % (
-            exception_value,
-            raise_value_name
-        )
-    )
+    emit("%s = %s;" % (exception_value, raise_value_name))
     getReferenceExportCode(raise_value_name, emit, context)
 
     emitErrorLineNumberUpdateCode(emit, context)
     emit(
-        "RAISE_EXCEPTION_%s( &%s, &%s, &%s );" % (
+        "RAISE_EXCEPTION_%s( &%s, &%s, &%s );"
+        % (
             ("IMPLICIT" if implicit else "WITH_VALUE"),
             exception_type,
             exception_value,
-            exception_tb
+            exception_tb,
         )
     )
 
-    emit(
-        getFrameVariableTypeDescriptionCode(context)
-    )
+    emit(getFrameVariableTypeDescriptionCode(context))
 
     getGotoCode(context.getExceptionEscape(), emit)
 
@@ -400,48 +365,30 @@ def _getRaiseExceptionWithValueCode(raise_type_name, raise_value_name, implicit,
         context.removeCleanupTempName(raise_value_name)
 
 
-def _getRaiseExceptionWithTracebackCode(raise_type_name, raise_value_name,
-                                       raise_tb_name, emit, context):
-    exception_type, exception_value, exception_tb, _exception_lineno = \
-      context.variable_storage.getExceptionVariableDescriptions()
+def _getRaiseExceptionWithTracebackCode(
+    raise_type_name, raise_value_name, raise_tb_name, emit, context
+):
+    exception_type, exception_value, exception_tb, _exception_lineno = (
+        context.variable_storage.getExceptionVariableDescriptions()
+    )
 
-    emit(
-        "%s = %s;" % (
-            exception_type,
-            raise_type_name
-        )
-    )
+    emit("%s = %s;" % (exception_type, raise_type_name))
     getReferenceExportCode(raise_type_name, emit, context)
-    emit(
-        "%s = %s;" % (
-            exception_value,
-            raise_value_name
-        )
-    )
+    emit("%s = %s;" % (exception_value, raise_value_name))
     getReferenceExportCode(raise_value_name, emit, context)
 
-    emit(
-        "%s = (PyTracebackObject *)%s;" % (
-            exception_tb,
-            raise_tb_name
-        )
-    )
+    emit("%s = (PyTracebackObject *)%s;" % (exception_tb, raise_tb_name))
     getReferenceExportCode(raise_tb_name, emit, context)
 
     emit(
-        "RAISE_EXCEPTION_WITH_TRACEBACK( &%s, &%s, &%s);" % (
-            exception_type,
-            exception_value,
-            exception_tb
-        )
+        "RAISE_EXCEPTION_WITH_TRACEBACK( &%s, &%s, &%s);"
+        % (exception_type, exception_value, exception_tb)
     )
 
     # If anything is wrong, that will be used.
     emitErrorLineNumberUpdateCode(emit, context)
 
-    emit(
-        getFrameVariableTypeDescriptionCode(context)
-    )
+    emit(getFrameVariableTypeDescriptionCode(context))
 
     getGotoCode(context.getExceptionEscape(), emit)
 
