@@ -21,10 +21,7 @@
 
 
 template_metapath_loader_compiled_module_entry = """\
-{ "%(module_name)s", MOD_INIT_NAME( %(module_identifier)s ), 0, 0, NUITKA_COMPILED_MODULE },"""
-
-template_metapath_loader_compiled_package_entry = """\
-{ "%(module_name)s", MOD_INIT_NAME( %(module_identifier)s ), 0, 0, NUITKA_PACKAGE_FLAG },"""
+{ "%(module_name)s", MOD_INIT_NAME( %(module_identifier)s ), 0, 0, %(flags)s },"""
 
 template_metapath_loader_shlib_module_entry = """\
 { "%(module_name)s", NULL, 0, 0, NUITKA_SHLIB_FLAG },"""
