@@ -155,12 +155,6 @@ class StatementAssignmentVariable(StatementChildHavingBase):
         self.variable_trace = None
         self.inplace_suspect = None
 
-    def getDetail(self):
-        if self.variable is not None:
-            return "to variable %s" % self.variable
-        else:
-            return "to variable %s" % self.variable_name
-
     def getDetails(self):
         return {"variable": self.variable}
 
@@ -314,7 +308,7 @@ Assignment raises exception in assigned value, removed assignment.""",
                     result,
                     "new_statements",
                     """\
-Reduced assignment of %s from itself to mere access of it."""
+Lowered assignment of %s from itself to mere access of it."""
                     % variable.getDescription(),
                 )
             else:
@@ -474,12 +468,6 @@ class StatementDelVariable(StatementBase):
         del self.variable_trace
         del self.previous_trace
 
-    def getDetail(self):
-        if self.variable is not None:
-            return "to variable %s" % self.variable
-        else:
-            return "to variable %s" % self.variable_name
-
     def getDetails(self):
         return {
             "variable": self.variable,
@@ -632,9 +620,6 @@ class StatementReleaseVariable(StatementBase):
         del self.variable_trace
         del self.parent
 
-    def getDetail(self):
-        return "of variable %s" % self.variable
-
     def getDetails(self):
         return {"variable": self.variable}
 
