@@ -35,6 +35,7 @@ from nuitka.utils.FileOperations import (
     getSubDirectories,
     removeDirectory,
 )
+from nuitka.utils.SharedLibraries import locateDLL
 from nuitka.utils.Utils import isWin32Windows
 
 
@@ -51,6 +52,32 @@ class NuitkaPluginPyQtPySidePlugins(NuitkaPluginBase):
     def __init__(self):
         self.qt_dirs = {}
 
+    @staticmethod
+    def createPreModuleLoadCode(module):
+        """ Method called when a module is being imported.
+
+        Notes:
+            If full name equals "PyQt?" we insert code to include the dist
+            folder in the 'PATH' environment variable (on Windows only).
+
+        Args:
+            module: the module object
+        Returns:
+            Code to insert and descriptive text (tuple), or (None, None).
+        """
+        if not isWin32Windows():  # we are only relevant on Windows
+            return None, None
+
+        if module.getFullName() not in ("PyQt4", "PyQt5"):
+            return None, None  # not for us
+
+        code = """import os
+path = os.environ.get("PATH", "")
+if not path.startswith(__nuitka_binary_dir):
+    os.environ["PATH"] = __nuitka_binary_dir + ";" + path
+"""
+        return code, "Adding dist folder to 'PATH' environment variable."
+
     def getPyQtPluginDirs(self, qt_version):
         if qt_version in self.qt_dirs:
             return self.qt_dirs[qt_version]
@@ -123,7 +150,7 @@ if os.path.exists(guess_path):
                 yield qt_bin_dir
 
     def considerExtraDlls(self, dist_dir, module):
-        # pylint: disable=too-many-branches,too-many-locals
+        # pylint: disable=too-many-branches,too-many-locals,too-many-statements
         full_name = module.getFullName()
 
         if full_name in ("PyQt4", "PyQt5"):
@@ -288,6 +315,19 @@ if os.path.exists(guess_path):
                             shutil.copy(filename, os.path.join(dist_dir, basename))
 
             return result
+        elif full_name == "PyQt5.QtNetwork":
+            if not isWin32Windows():
+                dll_path = locateDLL("crypto")
+
+                if dll_path is None:
+                    dist_dll_path = os.path.join(dist_dir, os.path.basename(dll_path))
+                    shutil.copy(dll_path, dist_dll_path)
+
+                dll_path = locateDLL("ssl")
+                if dll_path is not None:
+                    dist_dll_path = os.path.join(dist_dir, os.path.basename(dll_path))
+
+                    shutil.copy(dll_path, dist_dll_path)
 
         return ()
 
