@@ -273,6 +273,7 @@ else:
 builtin_len_spec = BuiltinParameterSpecNoKeywords("len", ("object",), 0)
 builtin_dict_spec = BuiltinParameterSpec("dict", (), 0, "list_args", "dict_args")
 builtin_len_spec = BuiltinParameterSpecNoKeywords("len", ("object",), 0)
+builtin_any_spec = BuiltinParameterSpecNoKeywords("any", ("object",), 0)
 
 if python_version < 370:
     builtin_tuple_spec = BuiltinParameterSpec("tuple", ("sequence",), 1)
@@ -506,6 +507,8 @@ builtin_divmod_spec = BuiltinParameterSpecNoKeywords("divmod", ("left", "right")
 
 
 def extractBuiltinArgs(node, builtin_spec, builtin_class, empty_special_class=None):
+    # Many cases to deal with, pylint: disable=too-many-branches
+
     try:
         kw = node.getCallKw()
 
@@ -572,6 +575,8 @@ def extractBuiltinArgs(node, builtin_spec, builtin_class, empty_special_class=No
 
     # Using list reference for passing the arguments without names,
     result = builtin_class(*args_list, source_ref=node.getSourceReference())
-    result.setCompatibleSourceReference(node.getCompatibleSourceReference())
+
+    if python_version < 380:
+        result.setCompatibleSourceReference(node.getCompatibleSourceReference())
 
     return result
