@@ -42,6 +42,7 @@ from nuitka.utils.FileOperations import (
 )
 
 from .SearchModes import (
+    SearchModeAll,
     SearchModeBase,
     SearchModeByPattern,
     SearchModeCoverage,
@@ -282,6 +283,14 @@ def decideFilenameVersionSkip(filename):
     if filename.endswith("36.py") and _python_version < "3.6":
         return False
 
+    # Skip tests that require Python 3.7 at least.
+    if filename.endswith("37.py") and _python_version < "3.7":
+        return False
+
+    # Skip tests that require Python 3.8 at least.
+    if filename.endswith("38.py") and _python_version < "3.8":
+        return False
+
     return True
 
 
@@ -304,7 +313,9 @@ def _removeCPythonTestSuiteDir():
             raise
 
 
-def compareWithCPython(dirname, filename, extra_flags, search_mode, needs_2to3):
+def compareWithCPython(
+    dirname, filename, extra_flags, search_mode, needs_2to3, on_error=None
+):
     """ Call the comparison tool. For a given directory filename.
 
     The search mode decides if the test case aborts on error or gets extra
@@ -347,8 +358,10 @@ def compareWithCPython(dirname, filename, extra_flags, search_mode, needs_2to3):
     _removeCPythonTestSuiteDir()
 
     if result != 0 and result != 2 and search_mode.abortOnFinding(dirname, filename):
-        my_print("Error exit!", result)
-        sys.exit(result)
+        if on_error is not None:
+            on_error(dirname, filename)
+
+        search_mode.onErrorDetected("Error exit! %s" % result)
 
     if converted:
         os.unlink(path)
@@ -372,8 +385,7 @@ def checkCompilesNotWithCPython(dirname, filename, search_mode):
         result = 2
 
     if result != 1 and result != 2 and search_mode.abortOnFinding(dirname, filename):
-        my_print("Error exit!", result)
-        sys.exit(result)
+        search_mode.onErrorDetected("Error exit! %s" % result)
 
 
 def checkSucceedsWithCPython(filename):
@@ -837,6 +849,7 @@ def checkReferenceCount(checked_function, max_rounds=10):
 
 
 def createSearchMode():
+    all_mode = len(sys.argv) > 1 and sys.argv[1] == "all"
     search_mode = len(sys.argv) > 1 and sys.argv[1] == "search"
     resume_mode = len(sys.argv) > 1 and sys.argv[1] == "resume"
     only_mode = len(sys.argv) > 1 and sys.argv[1] == "only"
@@ -844,8 +857,9 @@ def createSearchMode():
     coverage_mode = len(sys.argv) > 1 and sys.argv[1] == "coverage"
 
     if coverage_mode:
-
         return SearchModeCoverage()
+    elif all_mode:
+        return SearchModeAll()
     elif resume_mode:
         return SearchModeResume(sys.modules["__main__"].__file__)
     elif search_mode and start_at:
